import argparse
from . import cards
from pathlib import Path
collection = cards.Collection()
decklist = cards.Decklist()

# type checking
def deck_type(arg):
	if arg.isdigit(): arg = int(arg)
	if arg in decklist.keys():
		return decklist[arg]
	raise argparse.ArgumentTypeError('\n\nThat is not a deck. Use [-d] to list decks')
def card_type(arg):
	if arg.isdigit(): arg = int(arg)
	if arg in [card.id for card in collection]:
		return cards.Card(arg) 
	raise argparse.ArgumentTypeError('\n\nThat is not a valid card id. Use [-q] to search for a card id.')
def deck_or_card(arg):
	if arg.isdigit(): arg = int(arg)
	if arg in decklist.keys():
		return decklist[arg]
	if arg in [card.id for card in collection]:
		return cards.Card(arg) 
	raise argparse.ArgumentTypeError('\n\nThat is neither a deck nor a card.\n' + 
					 'Valid arguments are: \n' +
					 '1) A deck name \n' +
					 '2) A deck id (d will list deck ids) \n' +
					 '3) A card id (use q to search for a car id)')

# argument parsing
parser = argparse.ArgumentParser()
parser.set_defaults(decks = [], cards = [], decks_or_cards = [], func = 'statistics')
subparsers = parser.add_subparsers()
# commands which take a deck as an argument
line_add_parser = subparsers.add_parser('line_add',aliases=['1'], help='add a basic card quickly')
line_add_parser.add_argument('decks',type=deck_type,nargs='*')
line_add_parser.set_defaults(func = 'line_add')

add_parser = subparsers.add_parser('add',aliases=['a'], help='add a basic card')
add_parser.add_argument('decks',type=deck_type,nargs='*')
add_parser.set_defaults(func = 'add')

image_cloze_parser = subparsers.add_parser('image_cloze',aliases=['ic'], help='generate an image cloze card')
image_cloze_parser.add_argument('image_path',type=Path)
image_cloze_parser.add_argument('decks',type=deck_type,nargs='*')
image_cloze_parser.set_defaults(func = 'image_cloze')

query_parser = subparsers.add_parser('query',aliases=['q'], help='search collection for regexp')
query_parser.add_argument('pattern',nargs='?',default='.')
query_parser.add_argument('-i', '--index_only',action='store_true',help='only output the index of the match')
query_parser.set_defaults(func = 'query')
# TODO miscellaneous filter options (due_date/studied_time/creation_date/deleted) for the query
visual_query_parser = subparsers.add_parser('visual_query',aliases=['vq'], help='search collection for regexp and perform a command on the selected card')
visual_query_parser.add_argument('pattern',nargs='?',default='.')
visual_query_parser.set_defaults(func = 'visual_query')
# commands which take a deck or several cards as an argument 
study_parser = subparsers.add_parser('study',aliases=['s'], help='study the collection or selected deck')
study_parser.add_argument('decks_or_cards', type=deck_or_card, nargs='*')
study_parser.add_argument('--date', type=int, help='study as if today was [date]')
study_parser.set_defaults(func = 'study')
statistics_parser = subparsers.add_parser('statistics',aliases=['S'], help='statistics about the selected deck or card')
statistics_parser.add_argument('decks_or_cards', type=deck_or_card, nargs='*')
statistics_parser.set_defaults(func = 'statistics')
# TODO miscellaneous options for more advanced statistics
edit_parser = subparsers.add_parser('edit',aliases=['e'], help='edit the selected deck or card')
edit_parser.add_argument('decks_or_cards',type=deck_or_card,nargs='*')
edit_parser.set_defaults(func = 'edit')
delete_parser = subparsers.add_parser('delete',aliases=['x'], help='delete the selected deck or card')
delete_parser.add_argument('decks_or_cards',type=deck_or_card, nargs='*')
delete_parser.set_defaults(func = 'delete')
# commands which take no arguments
list_decks_parser = subparsers.add_parser('list_decks',aliases=['d'], help='list decks')
list_decks_parser.set_defaults(func = 'list_decks')
visual_decks_parser = subparsers.add_parser('visual_decks',aliases=['vd'], help='select a deck and perform a command on it')
visual_decks_parser.set_defaults(func = 'visual_decks')
purge_parser = subparsers.add_parser('purge',aliases=['p'], help='permanently delete all cards scheduled for deletion')
purge_parser.set_defaults(func = 'purge')
# commands which take a path as an argument (import / backup)
backup_parser = subparsers.add_parser('backup',aliases=['b','export'], help='backup all cards')
backup_parser.add_argument('backup_dest',type=Path)
backup_parser.add_argument('decks_or_cards',type=deck_or_card,nargs='*') 
backup_parser.set_defaults(func = 'backup')
# TODO allow for selective export (e.g. we could query, get the first field, and pipe to export)
import_parser = subparsers.add_parser('import',aliases=['i'], help='import a collection of cards')
import_parser.add_argument('import_dest',type=Path)
import_parser.add_argument('-o','--overwrite',action='store_true',help='overwrite the existing collection')
import_parser.set_defaults(func = 'import_collection')
#
args = parser.parse_args()
for e in args.decks_or_cards:
	assert type(e) is cards.Card or type(e) is cards.Deck
	if type(e) is cards.Card:
		args.cards.append(e)
	if type(e) is cards.Deck:
		args.decks.append(e)	



# a basic card reviewer making use of cat and readchar
import subprocess
import readchar
import datetime

from . import video
# we could refactor this into the card class and make
# a function add_grade instead of add_history
today = datetime.date.today() 

grade_dict = {'-': -3,
	      '@': -2,
	      'x': -1,
	      'q': -1,
	      'u': -1,
	      'h': 0,
	      'w': 0,
	      '1': 1,
	      '2': 2,
	      '3': 3,
	      '4': 4,
	      ' ': 3,
	      '\r': 3,
	      '\n': 3}

cont_dict = {'-': 1,
	     '@': 1,
	     'x': 1,
	     'q': 0,
	     'u': -1,
	     'h': -1,
	     'w': 0,
	     '1': 1,
	     '2': 1,
	     '3': 1,
	     '4': 1,
	     ' ': 1,
	     '\r': 1,
	     '\n': 1}

assert cont_dict.keys() == grade_dict.keys()

def review(card,mode):
	# clear screen
	# show front
	# wait for keystroke / check if mode is preview
	# show back
	# call editor(?) / add a single card
	# assign grade (incl. schedule for suspend, bury, deletion)
	# call scheduler
	# return continue value

	# save the screen; tput smcup generates an escape code
	# which tells the terminal to save the screen
	# processes like vim, top, and less use this to recover the terminal at the end

	start = datetime.datetime.now()  # begin card timer

	subprocess.run(['tput','smcup'])
	subprocess.run(['tput','civis'])

	image_back = None
	def show_card():
		global image_back
		subprocess.run(['clear'])
		subprocess.run(['tput','civis'])

		# text
		subprocess.run(['cat',card.path / 'front'])  # view frontside

		# audio
		audio_front_path = card.path / 'audio_front'
		audio_front = Recording(audio_front_path) if audio_front_path.exists() else None
		if audio_front: audio_front.play()

		# images
		image_front_path = card.path / 'image_front'
		image_front = video.DisplayImage(image_front_path) if image_front_path.exists() else None
		if image_front: image_front.show()

		# card flip
		if mode == 'review':
			char = readchar.readchar() # later I might allow adding and quitting on the frontside

		# backside text
		subprocess.run(['echo','\n\n']) # dividing space
		subprocess.run(['cat',card.path / 'back'])  # view backside

		# erase frontside audio/images
		if audio_front: audio_front.stop()
		if image_front: image_front.close()
			
		# audio
		audio_back_path = card.path / 'audio_back'
		audio_back = Recording(audio_back_path) if audio_back_path.exists() else None
		if audio_back: audio_back.play()

		# images
		image_back_path = card.path / 'image_back'
		image_back = video.DisplayImage(image_back_path) if image_back_path.exists() else None
		if image_back: image_back.show()

	show_card()

	char = None
	while char not in grade_dict.keys():
		if char in ['e','f','b','*']:
			subprocess.run(['tput','rmcup']) # restore terminal state so that vim does not overwrite smcup
			card.edit(mode=char)
			subprocess.run(['tput','smcup']) 
			show_card()
		char = readchar.readchar()

	grade = grade_dict[char]
	cont = cont_dict[char]

	stop = datetime.datetime.now()  # stop card timer
	elapsed_time = min(120, (stop - start).seconds)
	
	card.add_history(today, elapsed_time, grade)

	# restore terminal
	if image_back: image_back.close()
	subprocess.run(['tput','rmcup'])
	subprocess.run(['tput','cvvis'])
	
	return cont
	# -1 means go back to previous card
	# 0 means exit
	# 1 means continue



#imports
import string
import datetime
import os
import shutil
from pathlib import Path

from . import reviewers, editors, schedulers 
from . import browser

vinca_path = Path(__file__).parent  # /path/to/vinca
cards_path = vinca_path / 'cards'
decks_file = vinca_path / 'decks.txt'
tags_file = vinca_path / 'tags.txt'

class Deck:
	def __init__(self,name,
	tags_include=[], tags_exclude=[], tags_create=[], idx=None):
		self.name = name
		self.tags_include = tags_include
		self.tags_exclude = tags_exclude
		self.tags_create = tags_create
		self.idx = idx
	def __str__(self):
		l = ['+' + t for t in self.tags_include]
		l += ['-' + t for t in self.tags_exclude]
		l += ['=' + t for t in self.tags_create]
		return f'{self.name:16s}' + ' '.join(l)

class Card:
	def __init__(self, cid=None, create=False):
		if not create:
			assert cid and type(cid)==int
			self.id = cid
			self.path = cards_path/str(cid)
			assert self.path.exists()
		elif create:
			old_cids = [int(x.name) for x in cards_path.iterdir()]
			self.id = max(old_cids) + 1 if old_cids else 100 
			self.path = cards_path/str(self.id)
			self.path.mkdir()
			self.make_metadata()

	def make_metadata(self):
		for f in ['history','due_date','tags','config']:
			(self.path/f).touch()

	def make_config(self, editor, reviewer, scheduler):
		(self.path/'config').write_text(f'{editor} {reviewer} {scheduler}')

	def set_due_date(self, date):
		assert type(date)==datetime.date or date in ['deleted','suspended']
		(self.path/'due_date').write_text(str(date))

	def add_tag(self, tag):
		tags = self.tags | {tag}
		(self.path/'tags').write_text(' '.join(tags))

	def remove_tag(self, tag):
		tags = self.tags - {tag}
		(self.path/'tags').write_text(' '.join(self.tags))

	def add_history(self, date, time, grade):
		assert type(date) == datetime.date
		assert type(grade) == int and type(time) == int
		with (self.path/'history').open('a') as f:
			f.write(f'{date} {time} {grade}\n')

	def undo_history(self):
		new_history = self.history[:-1]
		(self.path/'history').open('r+').truncate() # clear the history
		for date, time, grade in new_history:
			self.add_history(date, time, grade) 
			

	def auxfile(self, filename, mode='r'):
		# aux file stands for auxilliary file
		# this is the interface for all file manipulation
		assert filename in self.auxfiles
		return (self.path/filename).open(mode)

	def make_auxfile(self, filename):
		(self.path/filename).touch()

	def review(self, mode='review'):
		return reviewers.review(self, mode) 

	def edit(self, mode='e'):
		return editors.edit(self, mode) 

	def schedule(self):
		return schedulers.schedule(self) 

	def delete(self):
		self.set_due_date('deleted')

	@property
	def auxfiles(self):
		return [f.name for f in self.path.iterdir() if f.name not in ['tags','config','due_date','history']]

	# functools.cached_property could speed this up
	# the add and undo history commands would call an update
	@property
	def history(self):
		# read the history file into a list of entries of
		with open(f'{self.path}/history') as f:
			history = []
			for line in f.readlines():
				date, time, grade = line.split()
				date = datetime.date.fromisoformat(date)
				time, grade = int(time), int(grade)
				history.append((date, time, grade))
			return history

	@property
	def tags(self):
		return set((self.path/'tags').read_text().split())

	@property
	def due_date(self):
		dd = (self.path/'due_date').read_text()
		if dd in ['deleted', 'suspended']:
			return dd
		year, month, day = [int(x) for x in dd.split('-')]
		return datetime.date(year, month, day)

	@property
	def create_date(self):
		return self.history[0][0]
	@property
	def last_date(self):
		return self.history[-1][0]
	@property
	def last_grade(self):
		return self.history[-1][2]
	@property
	def last_interval(self):
		return self.history[-1][0] - self.history[-2][0]  

	@property
	def newQ(self):
		return len(self.history) == 1
	@property
	def deletedQ(self):
		return self.due_date == 'deleted'
	@property
	def suspendedQ(self):
		return self.due_date == 'suspended'

	@property
	def editor(self):
		editor, reviewer, scheduler = (self.path/'config').read_text().split()
		return editor
	@property
	def reviewer(self):
		editor, reviewer, scheduler = (self.path/'config').read_text().split()
		return reviewer
	@property
	def scheduler(self):
		editor, reviewer, scheduler = (self.path/'config').read_text().split()
		return scheduler

class Collection(list):
	def __init__(self):
		list.__init__(self)
		for card_path in cards_path.iterdir():
			cid = int(card_path.name)
			self.append(Card(cid))

	def add_card(self, cid):
		card = Card(cid)
		self.append(card)

	def cache_tags(self):
		tags = set()
		for card in self:
			tags |= card.tags
		tags_file.write_text('\n'.join(tags))
		


class Decklist(dict):
	def __init__(self):
		dict.__init__(self)
		lines = decks_file.read_text().splitlines()
		for i,line in enumerate(lines):
			name, *tags = line.split()
			include = [t[1:] for t in tags if t[0]=='+']
			exclude = [t[1:] for t in tags if t[0]=='-']
			create = [t[1:] for t in tags if t[0]=='=']

			deck = Deck(name,include,exclude,create,i)
			self[name] = deck
			self[i] = deck
	def __len__(self):
		return dict.__len__(self) // 2

	def delete(self, deck):
		del self[deck.name]
		del self[deck.idx]
		l = [str(d) for d in set(self.values())]
		decks_file.write_text('\n'.join(l))





import tkinter as Tk
from subprocess import run

TERMINAL_BACKGROUND = '#000000'

class ActiveWindow:
    def __init__(self):
        # use xdotool to query basic information about the active terminal's geometry
        out = run(['xdotool','getactivewindow','getwindowgeometry'], capture_output=True)
        # parse the output
	# and calculate handy references to the
	# active terminal's geometry
        out = str(out.stdout, encoding='utf-8')
        window_id, abs_pos, geometry = out.splitlines()
        self.left, self.top = abs_pos.split()[1].split(',')
        self.left, self.top = int(self.left), int(self.top)
        self.width, self.height = geometry.split()[1].split('x')
        self.width, self.height = int(self.width), int(self.height)
        self.right = self.left + self.width
        self.bottom = self.top + self.height
        self.center_x = self.left + self.width // 2
        self.center_y = self.top + self.height // 2

class DisplayImage:
    '''A simple class to draw an image to the screen.
    There are two methods: show and close.'''
    def __init__(self, image_path):
        self.image_path = image_path

    def show(self):
	# get geometry information about the active terminal
        aw = ActiveWindow()

        self.root = Tk.Tk()

        self.image = Tk.PhotoImage(file=self.image_path)
	# warning: images cannot be stored as local variables
	# of a function or else there is risk of them being
	# destroyed by premature garbage collection.

	# we want to center the image at the bottom of the terminal
	# with a 40 pixel margin on all sides
	# if the image is too big we will only see part of it
	# but the window will fit inside the active terminal
        img_height = self.image.height()
        img_width = self.image.width()
        margin = 40
        left = max(aw.left + margin, aw.center_x - img_width // 2)
        right = min(aw.right - margin, aw.center_x + img_width // 2)
        width = right - left
        bottom = aw.bottom - margin
        top = max(aw.top + margin, bottom - img_height)
        height = bottom - top
        self.root.geometry(f'{width}x{height}+{left}+{top}')

	# we want to draw the window
	# we do not want to let the window manager make decisions
        self.root.overrideredirect(True)

	# draw the image on a canvas
	# place our canvas to occupy the whole window
        self.canvas = Tk.Canvas(self.root, width=width, height=height)
        self.canvas.config(bg=TERMINAL_BACKGROUND)
        self.canvas.create_image(0,0, anchor=Tk.NW, image=self.image)
        self.canvas.place(x=-1,y=-1, height=height+2, width=width+2)

	# draw our window to the screen
        self.root.update()

	# (it is more common to see root.mainloop called
	#  to draw the window, but we choose to do the
	#  update call manually because there is no user
	#  interaction with the window.)

    def close(self):
        assert self.root
        self.root.destroy()




# A basic browser
# we are not trying to optimize anything
# a very easy way to speed up deck queues and statistics
# would be to provide the browser a sorted collection of cards
# the search could stop after due_date_end
import subprocess
import os

def filter(cards,
	   tags_include=None, tags_exclude=None, deck=None,
	   create_date_start=None, create_date_end=None,
	   seen_date_start=None, seen_date_end=None,
	   due_date_start=None, due_date_end=None,
	   last_difficulty=None,
	   editor=None,
	   reviewer=None,
	   scheduler=None,
	   deleted=False,
	   suspended=False,
	   new_only=False,
	   review_only=False):
	
	# python is weird: initializing a paremter to [] will cause bugs
	tags_include = tags_include if tags_include else []
	tags_exclude = tags_exclude if tags_exclude else []

	matches = []

	for card in cards:
		if deck:
			tags_include += deck.tags_include
			tags_exclude += deck.tags_exclude
		if tags_include and not any([t in tags_include for t in card.tags]):
			continue
		if tags_exclude and any([t in tags_exclude for t in card.tags]):
			continue
		if create_date_start and card.create_date < create_date_start:
			continue
		if create_date_end and card.create_date > create_date_start:
			continue
		if seen_date_start and card.seen_date < seen_date_start:
			continue
		if seen_date_end and card.seen_date > seen_date_start:
			continue
		if card.suspendedQ or card.deletedQ:
			continue
		if due_date_start and card.due_date < due_date_start:
			continue
		if due_date_end and card.due_date > due_date_end:
			continue
		
		if editor and card.editor != editor:
			continue
		if reviewer and card.reviewer != reviewer:
			continue
		if scheduler and card.scheduler != scheduler:
			continue

		if new_only and not card.newQ:
			continue
		if review_only and card.newQ:
			continue
		# TODO: pattern matching for small files using grep

		matches.append(card)

	# print(f'matches = {matches}')
	return matches

def search(cards, pattern):
	# This lets us search our collection for a regular expression
	# We use grep to do the actual work
	# We return a list of matches of the form (card, hit)

	matches = []

	for card in cards:
		if len(matches) > 12:
			break
		for auxfile in card.auxfiles:
			path = f'{card.path}/{auxfile}'
			
			# skip files larger than 1kb
			size = os.stat(path).st_size
			if size > 1000:
				continue
			hits = subprocess.run(['grep',pattern,path],capture_output=True)
			hits = hits.stdout
			hits = hits.splitlines()
			if hits:
				hit = hits[0]
				hit = str(hit, encoding='utf-8')
				matches.append((card, hit))
				break  # move on to the next card
	return matches
		 
			
		


	



# this is the base editor for creation of simple front/back plaintext cards
# TODO no filename
#      no buffer titles
#      no tildes
import subprocess
from pathlib import Path

vinca_path = Path(__file__).parent.parent.parent
tags_path = vinca_path / 'tags.txt'

path = Path(__file__).parent
vimrc_path = path / '.vimrc'

vimrc_modes = {'a': ['-c', 'startinsert'],
	       'e': [],
	       'f': [],
               'b': ['-c', 'wincmd W'],
               't': ['-c', 'wincmd j']}


def edit(card, mode):
	# possible modes are a, e, f, b
	# they stand for add, edit, front_edit, and back_edit
	implemented_modes = ['a','e','f','b','t']
	if mode not in implemented_modes:
		return

	if mode == 'a':
		card.make_auxfile('front')
		card.make_auxfile('back')

	assert 'front' in card.auxfiles
	assert 'back' in card.auxfiles

	# we are going to run vim...
	vim_cmd = ['vim']
	# to edit the fron and back sides of the cards...
	vim_cmd += ['-o',f'{card.path}/front',
	            f'{card.path}/back',f'{card.path}/tags']
	# using a vimrc file to make a few custom bindings...
	vim_cmd += ['-Nu', vimrc_path]
	# including tag autocompletion
	vim_cmd += ['-c', f'set dictionary={tags_path}']
	# with special options depending on the editing mode.
	vim_cmd += vimrc_modes[mode]
	# launch
	subprocess.run(vim_cmd)



# A dummy scheduler that always schedules for the next day
import datetime

def schedule(card):
	
	last_seen = card.history[-1][0]
	# [-1] means most recent entry in history
	# [-1][0] means to get the date (each entry is date, difficulty)

	# test interval is one day
	interval = datetime.timedelta(days=1)

	due_date = last_seen + interval

	card.set_due_date(due_date)

	# indicate that it is not necessary to review this card again today by putting it in the queue
	return 0




# functions
from pathlib import Path
from subprocess import run
import datetime
today = datetime.date.today()
study_day = today

import readchar  # 3rd party module for reading a character one at a time

from . import cards
col = cards.Collection()
col.cache_tags()
decklist = cards.Decklist()

from shutil import copytree, rmtree
from sys import exit

from . import editors, reviewers, schedulers
from . import browser

vinca_path = Path(__file__) # /path/to/vinca/__init__.py
vinca_path = vinca_path.parent # /path/to/vinca
cards_path = vinca_path / 'cards'
decks_path = vinca_path / 'decks.txt'
decks_vimrc = vinca_path / 'decks.vimrc'
tags_path = vinca_path / 'tags.txt'


def study(args):
	queue = []
	for deck in args.decks:
		queue += browser.filter(col, deck=deck, due_date_end=study_day)
	for card in args.cards:
		queue.append(card)
	done_queue = []
	while queue:
		card = queue.pop()
		cont = card.review()
		if cont == -1:
			card.undo_history()
			queue.append(card)
			if not done_queue:
				break
			prev_card = done_queue.pop()
			prev_card.undo_history()
			queue.append(prev_card)
		if cont == 0:
			card.undo_history()
			break
		if cont == 1:
			card.schedule()

def line_add(args):
	tags_create = []
	for deck in args.decks:
		tags_create += deck.tags_create 
	new_card = cards.Card(create=True)
	new_card.make_config('base','base','base')
	for tag in tags_create:
		new_card.add_tag(tag)
	# BAD: this reading should be implemented elsewhere
	(new_card.path/'front').write_text(input('Q:   '))
	(new_card.path/'back').write_text(input('A:   '))
	new_card.add_history(today, 0, 0)
	new_card.schedule()
def add_basic(args):
	tags_create = []
	for deck in args.decks:
		tags_create += deck.tags_create 
	new_card = cards.Card(create=True)
	new_card.make_config('base','base','base')
	for tag in tags_create:
		new_card.add_tag(tag)
	new_card.edit(mode='a')  # add mode
	cont = new_card.review(mode='preview')
	return new_card, cont
def add(args):
	new_card, cont = add_basic(args)
	new_card.schedule()
def add_many(args): #TODO
	 prev_card = None
	 while True:	
		 new_card, cont = add_basic(args)
		 new_card.schedule()
		 if cont == -1:
			 prev_card.review(mode='preview')  # TODO does not support multilevel
			 prev_card.undo_history()  # we do not actually want to change the old card
		 if cont == 0:
			 break
		 if cont == 1:
			 prev_card = new_card
def image_cloze(args):
	pass
def statistics(args):
	for deck in args.decks:
		all_cards = browser.filter(col, deck=deck)
		due_cards = browser.filter(all_cards, due_date_end=study_day)
		print('\n' + deck.name)
		print(f'{len(all_cards)} total')
		print(f'{len(due_cards)} due today')
	for card in args.cards:
		print(f'\nCard #{card.id}')
		print(f'Due: {card.due_date}')
		print(f'Tags: {" ".join(card.tags)}')
		print(f'Date        Time   Grade')
		hist_lines = [f'{date} {time:5d} {grade:7d}' for date, time, grade in card.history]
		print('\n'.join(hist_lines))
	if not args.cards and not args.decks:
		due_cards = browser.filter(col, due_date_end=study_day)
		print(f'{len(col)} total')
		print(f'{len(due_cards)} due today')
def edit(args):
	if args.cards:
		for card in args.cards:
			card.edit()
		return
	vim_cmd = ['vim',decks_path] 
	vim_cmd += [f'+{args.decks[0].idx + 1}'] if args.decks else []
	vim_cmd += ['-Nu',decks_vimrc]
	vim_cmd += ['-c',f'set dictionary={tags_path}']
	run(vim_cmd)
def delete(args):
	for deck in args.decks: 
		decklist.delete(deck)
	for card in args.cards:
		card.delete()
arg_dict = {'a':add,
	    'A':add_many,
	    'e':edit,
	    'x':delete,
	    's':study,
	    'S':statistics,
	    '1':line_add}
		

# deck manipulation
def list_decks(args):
	for i in range(len(decklist)):
		deck = decklist[i]
		print(deck.idx, deck, sep='\t')
def visual_decks(args):
	n = len(decklist)
	selected = 0
	run(['tput','civis'])
	print('\n'*(n-1)) # move cursor down n lines
	while True:
		for i in range(n):
			run(['tput','cuu1']) # move up n lines
		for i in range(n):
			# print the decks with the selected deck highlighted
			if i == selected:
				run(['tput','smso'])
			print(decklist[i])
			run(['tput','sgr0'])
		#get key
		k = readchar.readchar()
		if k == 'j' and selected < n - 1:
			selected += 1
		if k == 'k' and selected > 0:
			selected -= 1
		if k == 'q' or k == readchar.key.ESC:
			run(['tput','cvvis'])
			exit(0)
			break
		if k in arg_dict.keys():
			# erase the menu
			for i in range(n):
				run(['tput','cuu1']) # move up n lines
				run(['tput','el']) # and clear them as we go
			# make cursor visible
			run(['tput','cvvis'])
			# select the selected deck
			args.decks = [decklist[selected]]
			# the key sets the same variable as that option normally would
			cmd = arg_dict[k]
			cmd(args)
			break
def query(args):
	matches = browser.search(col, args.pattern)
	for card, hit in matches:
		if args.index_only:
			print(card.id)
		else:
			print(card.id, hit[:50], sep='\t')
def visual_query(args):
	matches = browser.search(col, args.pattern)
	if not matches:
		print('No matches.')
		exit(0)
	n = len(matches)
	selected = 0
	run(['tput','civis'])
	print('\n'*(n-1)) # move down n lines
	while True:
		for i in range(n):
			run(['tput','cuu1']) # move up n lines
		for i,(card,hit) in enumerate(matches):
			if i == selected:
				run(['tput','smso']) # standout attribute
			if card.deletedQ:
				run(['tput','dim'])
			print(f'{hit[:50]}')
			run(['tput','sgr0']) # normal attribute
		#get key
		k = readchar.readchar()
		if k == 'j' and selected < n - 1:
			selected += 1
		if k == 'k' and selected > 0:
			selected -= 1
		if k == 'q' or k == readchar.key.ESC:
			run(['tput','cvvis'])
			exit(0)
			break
		if k in arg_dict.keys():
			# for i in range(n):
			# 	run(['tput','cuu1']) # move up n lines
			# 	run(['tput','el']) # and clear them as we go
			#run(['tput','cvvis'])
			args.cards = [matches[selected][0]]
			cmd = arg_dict[k]
			cmd(args)
			run(['tput','civis'])
			#break

def purge(args):
	for card in col:
		if card.deletedQ:
			rmtree(card.path)	
# backup
def backup(args):
	copytree(cards_path, args.small_backup)
def import_collection(args):
	rmtree(cards_path)
	copytree(args.import_path, cards_path)



import codecs
import setuptools


setuptools.setup(
    name="vinca",
    version="1.1.43",
    author="Oscar Laird", # your name
    author_email="olaird25@gmail.com", # your email
    description="A simple spaced repetition system",
    long_description=codecs.open("README.md", encoding="utf-8").read(),
    long_description_content_type="text/markdown",
    license="MIT", # license type
    include_package_data = True,
    packages=setuptools.find_packages(),
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    python_requires='>=3.5', # python version
    install_requires=[ # dependecies package
        'readchar',
    ],
    entry_points = {
        'console_scripts': ['vinca=vinca.run:main']
    },
)
